#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains constants definitions for tpRigToolkit
"""

from __future__ import print_function, division, absolute_import

DEFAULT_SIDES = {'center': 'c', 'left': 'l', 'right': 'r'}
DEFAULT_SIDE = 'center'
DEFAULT_MIRROR_SIDE = 'right'
